/*
  Copyright (c) 2022  Kazuhiro Ito (kzhr@d1.dion.ne.jp)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
*/

#include <config.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"

void estdb_informer(const char *message, void *opaque){
  fputs("[ESTDB] ", verbose);
  fputs(message, verbose);
  fputc('\n', verbose);
  fflush(verbose);
}

void DEBUG (const char *message, ...) {
  if (debug) {
    va_list ap;
    fputs("[DEBUG] ", stderr);
    va_start(ap, message);
    vfprintf(debug, message, ap);
    va_end(ap);
    fputc('\n', debug);
    fflush(debug);
  }
}

void VERBOSE (const char *message, ...) {
  if (verbose) {
    va_list ap;
    va_start(ap, message);
    vfprintf(verbose, message, ap);
    va_end(ap);
    fputc('\n', verbose);
    fflush(verbose);
  }
}

char *snprintf_buffer(const char* restrict format, ...) {
  va_list ap;

  va_start(ap, format);
  int size = vsnprintf(NULL, 0, format, ap);
  va_end(ap);

  if (size < 0)
    goto snprintf_error;

  char* result = malloc((size_t)size + 1);
  if (!result) {
    fprintf(stderr, "Memory exhausted\n");
    goto error;
  }

  va_start(ap, format);
  if (vsnprintf(result, size + 1, format, ap) != size) {
    goto snprintf_error;
  }

 finish:
  va_end(ap);
  return result;

 snprintf_error:
  fprintf(stderr, "vsnprintf() error\n");
 error:
  if (result) free(result);
  va_end(ap);
  return NULL;
}

char *trim_spaces(char *string, bool trim_lf) {
  if (!string) return string;
  char* start = string;

  while (*start == ' ' || *start == '\t') start++;

  char* p = start + strlen(start) - 1;
  if (trim_lf) {
    while (p >= start && (*p == ' ' || *p == '\t' || *p == '\r' || *p == '\n'))
      p--;
  } else {
    while (p >= start && (*p == ' ' || *p == '\t')) p--;
  }
  *(p + 1) = 0;

  return start;
}

void hexdump (FILE *fp, const char *data, int length) {
  if (length > 0x10000) length = 0x10000;
  char line[81]; /*  = "00000 00 00 ... 00  ......." */
  int i;
  for (i = 0; i < length; i++) {
    /* fprintf(stderr, "counter is %d\n", i); */
    if (!(i % 16)) {
      memset(line, 0, 81);
      sprintf(line, "%05x ", i);
    }
    sprintf (line + 6 + (i % 16) * 3, "%02x ", (unsigned char)(data[i]));
    line[6 + 16 * 3 + 1 + (i % 16)] =
      (data[i] >= 0x20 && data[i] <= 0x7f) ? data[i] : '.';
    if ((i % 16) == 15) {
      line[6 + 16 * 3] = ' ';
      line[6 + 16 * 3 + 1 + 16] = '\n';
      fputs(line, fp);
    }
  }
  if (i % 16) {
    memset (line + 6 + (i % 16) * 3, ' ', (16 - (i % 16)) * 3 + 1);
    line[6 + 16 * 3 + 1 + (i % 16)] = '\n';
    fputs(line, fp);
  }
}
